#ifndef HPY_COMMON_RUNTIME_CTX_MODULE_H
#define HPY_COMMON_RUNTIME_CTX_MODULE_H

#include "hpy.h"

_HPy_HIDDEN HPy ctx_Module_Create(HPyContext ctx, HPyModuleDef *hpydef);

#endif /* HPY_COMMON_RUNTIME_CTX_MODULE_H */
